==> worker-2.14.1.tar.gz: worker-2.14.1/src/nmextlist.cc <==
/* nmextlist.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2006 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "nmextlist.hh"
#include "normalmode.h"

NMExtList::NMExtList()
{
  extlist = new List();
  files = dirs = 0;
  take_dirs = false;
}

NMExtList::~NMExtList()
{
  NM_extern_fe *efe;

  if ( extlist == NULL ) return;
  efe = (NM_extern_fe*)extlist->getFirstElement();
  while ( efe != NULL ) {
    delete efe;
    efe = (NM_extern_fe*)extlist->getNextElement();
  }
  delete extlist;
}

int NMExtList::getSize()
{
  return extlist->size();
}

int NMExtList::getRealSize()
{
  return ( files + ( ( take_dirs == true ) ? dirs : 0 ) );
}

void NMExtList::setTakeDirs( bool nv )
{
  take_dirs = nv;
}

int NMExtList::initEnum()
{
  return extlist->initEnum();
}

void NMExtList::closeEnum( int id )
{
  extlist->closeEnum( id );
}

NM_extern_fe *NMExtList::getFirstElement( int id )
{
  return (NM_extern_fe*)extlist->getFirstElement( id );
}

NM_extern_fe *NMExtList::getNextElement( int id )
{
  return (NM_extern_fe*)extlist->getNextElement( id );
}

int NMExtList::removeElement( NM_extern_fe *efe )
{
  if ( efe == NULL ) return 1;
  if ( efe->getDirFinished() != 0 ) dirs--;
  else files--;
  extlist->removeElement( efe );
  return 0;
}

void NMExtList::createExtList( NormalMode *nm, bool rec )
{
  if ( nm == NULL ) return;

  std::list<NM_specialsourceExt*> *splist;

  splist = new std::list<NM_specialsourceExt*>;
  nm->getSelFiles( splist, NormalMode::NM_GETFILES_SELORACT );

  createExtList( splist, rec );

  NormalMode::freeSelFiles( splist );
  delete splist;

  files = getEntriesRealSize();
  dirs = extlist->size() - files;
}

/*
 * createExtList
 *
 * will create a list of all involved entries depending on what type
 * of message is given.
 * At the moment it handles SPECIAL and DND
 */
void NMExtList::createExtList( NormalMode *nm, ActionMessage *msg, bool rec )
{
  if ( nm == NULL ) return;

  std::list<NM_specialsourceExt*> *splist;
  std::list<NM_specialsourceExt*>::iterator ite1;
  NM_specialsourceExt *ss1;
  const DNDMsg *dm;

  splist = new std::list<NM_specialsourceExt*>;

  switch ( msg->mode ) {
    case ActionMessage::AM_MODE_SPECIAL:
      ss1 = nm->getSpecialsourceForFE( msg->getFE() );
      if ( ss1 != NULL ) splist->push_back( ss1 );
      break;
    case ActionMessage::AM_MODE_DNDACTION:
      dm = msg->dndmsg;
      //      if ( dm->getSourceLister() == parentlister ) {
        ss1 = nm->getSpecialsourceForFE( dm->getFE() );
        if ( ss1 != NULL ) splist->push_back( ss1 );
        //      }
      break;
    case ActionMessage::AM_MODE_ONLYACTIVE:
      break;
    default:
      break;
  }

  createExtList( splist, rec );

  for ( ite1 = splist->begin(); ite1 != splist->end(); ite1++ ) {
    delete *ite1;
  }
  delete splist;

  files = getEntriesRealSize();
  dirs = extlist->size() - files;
}

int NMExtList::create_externfe_rec( const char *fullname, bool rec, const FileEntry *fe, int row )
{
  ArrayList *f1;
  int id;
  NM_extern_fe *efe;
  Verzeichnis *tverz;
  FileEntry *tfe;
  
  if((Datei::fileExistsExt(fullname)==Datei::D_FE_DIR)&&(rec==true)) {
    // create all subentries
    tverz=new Verzeichnis();
    if(tverz->readDir(fullname)==0) {
      f1=tverz->getFiles();
      if(f1!=NULL) {
        id=f1->initEnum();
        tfe=(FileEntry*)f1->getFirstElement(id);
        while(tfe!=NULL) {
          if(strcmp(tfe->name,"..")!=0) {
            create_externfe_rec( tfe->fullname, rec, NULL, -1 );
          }
          tfe=(FileEntry*)f1->getNextElement(id);
        }
        f1->closeEnum(id);
      }
    }
    delete tverz;
    // now append the dir
    efe=new NM_extern_fe(fullname,fe,row);
    efe->setDirFinished(1);  // this indicates the end of the recursive dir
                             // so it can be deactivated in the LV
    extlist->addElement(efe);
  } else {
    efe=new NM_extern_fe(fullname,fe,row);
    extlist->addElement(efe);
  }
  return 0;
}

int NMExtList::createExtList( std::list<NM_specialsourceExt*> *splist, bool rec )
{
  NM_specialsourceExt *ss1;
  std::list<NM_specialsourceExt*>::iterator ite1;

  if ( splist == NULL ) return -1;

  for ( ite1 = splist->begin(); ite1 != splist->end(); ite1++ ) {
    ss1 = *ite1;
    NMExtList::create_externfe_rec( ss1->entry()->fullname, rec, ss1->entry(), ss1->row );
  }
  return 0;
}

int NMExtList::getEntriesRealSize()
{
  int id = extlist->initEnum(),rs;
  NM_extern_fe *efe1;
  efe1=(NM_extern_fe*)extlist->getFirstElement(id);
  rs=0;
  while(efe1!=NULL) {
    if(efe1->getDirFinished()==0) rs++;
    efe1=(NM_extern_fe*)extlist->getNextElement(id);
  }
  extlist->closeEnum(id);
  return rs;
}

==> worker_2.16.0.orig.tar.gz: worker-2.16.0/src/nmextlist.cc <==
/* nmextlist.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2007 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "nmextlist.hh"
#include "normalmode.h"

NMExtList::NMExtList()
{
  extlist = new List();
  files = dirs = 0;
  take_dirs = false;
}

NMExtList::~NMExtList()
{
  NM_extern_fe *efe;

  if ( extlist == NULL ) return;
  efe = (NM_extern_fe*)extlist->getFirstElement();
  while ( efe != NULL ) {
    delete efe;
    efe = (NM_extern_fe*)extlist->getNextElement();
  }
  delete extlist;
}

int NMExtList::getSize()
{
  return extlist->size();
}

int NMExtList::getRealSize()
{
  return ( files + ( ( take_dirs == true ) ? dirs : 0 ) );
}

void NMExtList::setTakeDirs( bool nv )
{
  take_dirs = nv;
}

int NMExtList::initEnum()
{
  return extlist->initEnum();
}

void NMExtList::closeEnum( int id )
{
  extlist->closeEnum( id );
}

NM_extern_fe *NMExtList::getFirstElement( int id )
{
  return (NM_extern_fe*)extlist->getFirstElement( id );
}

NM_extern_fe *NMExtList::getNextElement( int id )
{
  return (NM_extern_fe*)extlist->getNextElement( id );
}

int NMExtList::removeElement( NM_extern_fe *efe )
{
  if ( efe == NULL ) return 1;
  if ( efe->getDirFinished() != 0 ) dirs--;
  else files--;
  extlist->removeElement( efe );
  return 0;
}

void NMExtList::createExtList( NormalMode *nm, bool rec )
{
  if ( nm == NULL ) return;

  std::list<NM_specialsourceExt*> *splist;

  splist = new std::list<NM_specialsourceExt*>;
  nm->getSelFiles( splist, NormalMode::NM_GETFILES_SELORACT );

  createExtList( splist, rec );

  NormalMode::freeSelFiles( splist );
  delete splist;

  files = getEntriesRealSize();
  dirs = extlist->size() - files;
}

/*
 * createExtList
 *
 * will create a list of all involved entries depending on what type
 * of message is given.
 * At the moment it handles SPECIAL and DND
 */
void NMExtList::createExtList( NormalMode *nm, ActionMessage *msg, bool rec )
{
  if ( nm == NULL ) return;

  std::list<NM_specialsourceExt*> *splist;
  std::list<NM_specialsourceExt*>::iterator ite1;
  NM_specialsourceExt *ss1;
  const DNDMsg *dm;

  splist = new std::list<NM_specialsourceExt*>;

  switch ( msg->mode ) {
    case ActionMessage::AM_MODE_SPECIAL:
      ss1 = nm->getSpecialsourceForFE( msg->getFE() );
      if ( ss1 != NULL ) splist->push_back( ss1 );
      break;
    case ActionMessage::AM_MODE_DNDACTION:
      dm = msg->dndmsg;
      //      if ( dm->getSourceLister() == parentlister ) {
        ss1 = nm->getSpecialsourceForFE( dm->getFE() );
        if ( ss1 != NULL ) splist->push_back( ss1 );
        //      }
      break;
    case ActionMessage::AM_MODE_ONLYACTIVE:
      break;
    default:
      break;
  }

  createExtList( splist, rec );

  for ( ite1 = splist->begin(); ite1 != splist->end(); ite1++ ) {
    delete *ite1;
  }
  delete splist;

  files = getEntriesRealSize();
  dirs = extlist->size() - files;
}

int NMExtList::create_externfe_rec( const char *fullname, bool rec, const FileEntry *fe )
{
  NM_extern_fe *efe;
  Verzeichnis *tverz;
  
  if((Datei::fileExistsExt(fullname)==Datei::D_FE_DIR)&&(rec==true)) {
    // create all subentries
    tverz=new Verzeichnis();
    if ( tverz->readDir( fullname ) == 0 && tverz->dirOpened() == true ) {
        for ( Verzeichnis::verz_it tfe_it1 = tverz->begin();
              tfe_it1 != tverz->end();
              tfe_it1++ ) {
            FileEntry *tfe = tfe = *tfe_it1;
            if ( strcmp( tfe->name, ".." ) != 0 ) {
                create_externfe_rec( tfe->fullname, rec, NULL );
            }
        }
    }
    delete tverz;
    // now append the dir
    efe = new NM_extern_fe( fullname, fe);
    efe->setDirFinished(1);  // this indicates the end of the recursive dir
                             // so it can be deactivated in the LV
    extlist->addElement(efe);
  } else {
    efe = new NM_extern_fe( fullname, fe );
    extlist->addElement(efe);
  }
  return 0;
}

int NMExtList::createExtList( std::list<NM_specialsourceExt*> *splist, bool rec )
{
  NM_specialsourceExt *ss1;
  std::list<NM_specialsourceExt*>::iterator ite1;

  if ( splist == NULL ) return -1;

  for ( ite1 = splist->begin(); ite1 != splist->end(); ite1++ ) {
    ss1 = *ite1;
    NMExtList::create_externfe_rec( ss1->entry()->fullname, rec, ss1->entry() );
  }
  return 0;
}

int NMExtList::getEntriesRealSize()
{
  int id = extlist->initEnum(),rs;
  NM_extern_fe *efe1;
  efe1=(NM_extern_fe*)extlist->getFirstElement(id);
  rs=0;
  while(efe1!=NULL) {
    if(efe1->getDirFinished()==0) rs++;
    efe1=(NM_extern_fe*)extlist->getNextElement(id);
  }
  extlist->closeEnum(id);
  return rs;
}
